###############################################################################
#### R file: CorrectTMA.R  
####
#### This program is for evaluating the marginal density and distribution 
#### functions in TMA(2;1,1) models and possible for invertible regions   
#### The model: y_t=(e_t + phi*e_{t-1})I(y_{t-1}>=r)                        
####                (e_t + psi*e_{t-1})I(y_{t-1}<r)                         
####
#### Coded by: Dong Li
####
#### ## Reference:
#### Li, D. and Qiu, J. (2020).
####   The marginal density of a TMA(1) process. 
####   Journal of Time Series Analysis, 41, 476--484.
####
#### De Gooijer, J.G. (1998).
####   On threshold moving-average models. 
####   Journal of Time Series Analysis, 19(1), 1--18.
###############################################################################
library(matlab) # calling for "meshgrid"
###############################################################################
K.fun=function(x,y,par){
# This function is called for the meshgrid, z=(x,y)
# par=(phi, psi, r) is the parameter
# The error is N(0, 1), which can be changed.
#
if (par[1]>0 && par[2]>0){
  f=pnorm(x/par[1])-pnorm(min(x/par[1], par[3]-y))+pnorm(min(x/par[2], par[3]-y))
  }
else if (par[1]>0 && par[2]==0){
  f=pnorm(x/par[1])-pnorm(min(x/par[1], par[3]-y))+pnorm(par[3]-y)*(x>=0)
  }
else if (par[1]>0 && par[2]<0){
  f=pnorm(x/par[1])-pnorm(min(x/par[1], par[3]-y))+pnorm(par[3]-y)-pnorm(min(x/par[2], par[3]-y))
  }
else if (par[1]==0 && par[2]>0){
  f=(x>=0)*(1-pnorm(par[3]-y))+pnorm(min(x/par[2], par[3]-y))
  }
else if (par[1]==0 && par[2]==0){f=(x>=0)}
else if (par[1]==0 && par[2]<0){
  f=(x>=0)*(1-pnorm(par[3]-y))+pnorm(par[3]-y)-pnorm(min(x/par[2], par[3]-y))
  }
else if (par[1]<0 && par[2]>0){
  f=1-pnorm(max(x/par[1], par[3]-y))+pnorm(min(x/par[2], par[3]-y))
  }
else if (par[1]<0 && par[2]==0){
  f=1-pnorm(max(x/par[1], par[3]-y))+(x>=0)*pnorm(par[3]-y)
  }
else {
   f=1-pnorm(max(x/par[1], par[3]-y))+pnorm(par[3]-y)-pnorm(min(x/par[2], par[3]-y))
  }
return(f)
}
###############################################################################
###########         Evaluate the delta's distribution         #################
###############################################################################
Delta.dist=function(x,par){
n = length(x)
y = (c(min(x)-1,x)+c(x, max(x)+1))/2
m = length(y)
A = matrix(0,nrow=n,ncol=m)
  for (i in 1:n){
    A[i,]=sapply(y,K.fun,x=x[i],par=par)
  }
K  = A[,1:n]-A[,2:(n+1)]
a  = A[,n+1]
dd = solve(eye(n)-K)%*%a
return(dd)
}
###############################################################################
###############################################################################
g.density=function(x,par){
  G      = Delta.dist(x,par)
  diff.G = c(G,1)-c(0,G)
  n      = length(x)
  y      = (c(min(x)-1,x)+c(x, max(x)+1))/2
  z      = meshgrid(x,y)
  Den.matrix = t(dnorm(z$x-z$y))
  den.hat    = Den.matrix%*%diff.G
return(den.hat)
}
################################################################################

################################################################################
############  Next is the De Gooijer's method ##################################
################################################################################
int.fun=function(x,y,sigma,rho){
f=exp(-(x-rho*y)^2/(2*sigma*(1-rho^2))-y^2/(2*sigma))/(sigma*2*pi*sqrt(1-rho^2))
return(f)
}
##############################################################################
de.f.den=function(x,par){
sp   = 1+par[1]^2
sn   = 1+par[2]^2
rhop = par[1]/sqrt(sp)
rhon = par[2]/sqrt(sn)
#adfactor=1-pnorm(par[2]/sqrt(sp))+pnorm(par[2]/sqrt(sn))
f1 = integrate(int.fun, lower=par[3], x=x,upper=Inf,sigma=sp,rho=rhop)$value
f2 = integrate(int.fun, lower=-Inf, x=x,upper=par[3],sigma=sn,rho=rhon)$value
f=(f1+f2)#/adfactor
return(f)
}
##############################################################################
##############################################################################
de.gooijer=function(x,par){
sp   = sqrt(1+par[1]^2)
sn   = sqrt(1+par[2]^2)
rhop = par[1]/sp
rhon = par[2]/sn
f1   = exp(-x^2/(2*sp^2))*pnorm(rhop*x/(sp*sqrt(1-rhop^2)))/sqrt(2*pi*sp^2)
f2   = exp(-x^2/(2*sn^2))*pnorm(-rhon*x/(sn*sqrt(1-rhon^2)))/sqrt(2*pi*sn^2)
f    = f1+f2
return(f)
}
##############################################################################
##############################################################################
##############################################################################
##############         Plot the figures                 ######################
##############################################################################
##############################################################################
# par=c(0.9,-0.7,0)
par = c(0.5,-0.5,0)  # last entry is threshold value

h = 0.02
x = seq(-4,5,by=h)
g = g.density(x,par) ## Our method
Gooijer.den = sapply(x, de.f.den,par=par) ## De Gooijer method
##################################################
plot(x, g, type="l",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,col="blue",
     xlab="x",ylab="",ylim=c(0,0.4))
abline(h=c(0,1),lty=2)
lines(x,Gooijer.den,lty=2,col="red",lwd=3)
legend(-4,0.4,c("Ours", "De Gooijer"),lty=c(1,2),cex=2,col=c("blue","red"),lwd=3)
##################################################


##################################################
###############   The End   ######################
##################################################


##################################################
m       = length(x)
ourdist = c(cumsum((g[2:m]+g[1:(m-1)]))*h/2,1)
goodist = c(cumsum((Gooijer.den[2:m]+Gooijer.den[1:(m-1)]))*h/2,1)

plot(x, ourdist, type="l",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,col="blue",
  xlab="x",ylab="",ylim=c(0,1),xlim=c(-4,4))
abline(h=c(0,1,ourdist[x==par[3]]),v=par[3],lty=2)
lines(x,goodist,lty=2,col="red",lwd=3)
legend(-4,1,c("Ours", "De Gooijer"),lty=c(1,2),cex=2,col=c("blue","red"),lwd=3)
##################################################
#rzero=sapply(x,de.gooijer,par=par)
#lines(x,rzero,lty=4,col="cyan",lwd=3)
##############################################################################

##############################################################################
##############         Plot the invertible region       ######################
##############################################################################
##############################################################################
Fzero   = function(phi,psi,r){
par     = c(phi,psi,r)
h       = 0.02
x       = c(seq(r-6,r-h,by=h), r, seq(r+h,r+6,by=h))
g       = g.density(x,par)
ourdist = cumsum((g[2:m]+g[1:(m-1)]))*h/2
F0      = ourdist[x==par[3]]
return(F0)
}
##############################################################################
##############################################################################
Find.PSI=function(phi,r){
psiupper = 20
psilower = 0.1

 while (psiupper-psilower>1e-4){
   aup  = Fzero(phi,psiupper,r)
   amid = Fzero(phi,(psiupper+psilower)/2,r)
   alow = Fzero(phi,psilower,r)
   Fup  = abs(phi)^(1-aup)*abs(psiupper)^aup
   Fmid = abs(phi)^(1-amid)*abs((psiupper+psilower)/2)^amid
   Flow = abs(phi)^(1-alow)*abs(psilower)^alow
   if (Fmid>1){psiupper=(psiupper+psilower)/2}
   else{psilower=(psiupper+psilower)/2}
 }
return((psiupper+psilower)/2)
}
##############################################################################
r=0
phi=seq(0.1,1,by=0.02)
psi=sapply(phi,Find.PSI,r=0)

plot(phi,psi,type="l",las=1,lwd=3,cex.axis=1.3,cex.lab=1.3,
   xlab=expression(phi),ylab=expression(psi))














