%  File: pdf_Gaussian_WN_discretization.m
%
%  Input file: stationary.m (see pdf_ARCH_Gaussian_discretization.m)
%
%  Gaussian innovations
%
%  Support of the invariant probability
%  for rho1=0.4  support = [0,4/(1-0.4)]=[0,6.6667]=[-7,7]
%      rho1=0.5          = [0,4/(1-0.5)]=[-8,8]
%      rho1=0.6          = [0,4/(1-0.6)]=[-10,10] 
%      rho1=0.7          = [0,4/(1-0.7)]=[0,13.33]=[-14,14] 
%      rho1=0.8          = [0,4/(1-0.8)]=[-20,20] 
%      rho1=0.9          = [0,4/(1-0.9)]=[-40,40] 
%
% Reference:
%  Herve and Ledoux (2020). State-discretization of 
%    V-geometrically ergodic Markov chains and convergence to
%    stationarity, Methodology and Computing in Applied Probability,
%    22: 905--925.
% 
% Note: To reduce the heavy computations used to obtain B_{k}, note that
%       the terms below the main diagonal are zero.
%
% OUTPUT:
% Non-central moments
%    0.0000    1.3131    0.0001    5.2303
% Central moments
%    0.0000    1.3131   -0.0000    3.0332
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

% Set the AR(1) parameter: 
rho1 = 0.5;

% Define the density function of the errors, here N(mu,sigma)
sigma = 1;
mu    = 0;
nu    = @(x) pdf('normal',x,mu,sigma);
% fplot(nu,[-4,4]);

% Value of $delta_k$
deltak = 0.02;

% Values of kmin and kmax; support interval
kmin = -14;
kmax =  14;
qmax = (kmax-kmin)/deltak;

% grid points
xk = zeros(qmax+1);
for i=1:qmax+1
    xk(i) = kmin+(i-1)*deltak; % initial step 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct the [1401 * 1401] "B_k" matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization 
Bk = zeros(qmax+1,qmax+1);
% All rows and columns except the last one

for i=1:qmax      % Define the fcts "nu_{i,k}" 
    nuik = @(y) min(nu(y-rho1*xk(i)), nu(y-rho1*xk(i+1))); % AR(1)
    for j =1:qmax
        Bk(i,j) = quad(nuik,xk(j),xk(j+1));
    end
end

%% Choose the column j_{0}\in {1,...,qkmax} and set column j_{0} of B_{k} 
j0 = floor(qmax/2);
for i=1:qmax   
    Bk(i,j0+1) = Bk(i,j0+1) + 1 - quad(nuik,kmin,kmax); 
end
Bk(qmax+1,j0+1) = 1;  

% Compute the invariant probabilities B_{k}
pik = stationary(Bk);  % [1 * 1401]

% Define the pdf "p_{k}"
pk    = [];  

for y = kmin:(kmax-kmin)/100:kmax  
    % Compute "p_{k}(y)"
    val = 0;
    for i=1:qmax   %) Define the fcts nu_{i,k}
      nuik = @(y) min(nu(y-rho1*xk(i)), nu(y-rho1*xk(i+1)));  % AR(1)
      val  = val + pik(i)*nuik(y);
    end
    pk = [pk,val];
end

x = kmin:(kmax-kmin)/100:kmax;  
%%%%%%%%%%% Normalization of "p_{k}" %%%%%%%%%%%%
nor = trapz(pk);
pk2 = (pk/nor);

%% Non-central moments
nu_1 = 0;
nu_2 = 0;
nu_3 = 0;
nu_4 = 0;
for u =1:length(x)
  nu_1 = nu_1 + x(u)*pk2(u)*100/length(x);
  nu_2 = nu_2 + x(u)*x(u)*pk2(u)*100/length(x);
  nu_3 = nu_3 + x(u)*x(u)*x(u)*pk2(u)*100/length(x);
  nu_4 = nu_4 + x(u)*x(u)*x(u)*x(u)*pk2(u)*100/length(x);  
end

%% Central moments

VAR = nu_2 - nu_1^(2);
sd  = sqrt(VAR);

mu3 = nu_3 - 3* nu_2 * nu_1 + 2*(nu_1)^(3);
mu4 = nu_4 - 4* nu_3 * nu_1 + 6*nu_2*(nu_1)^(2) - 3*(nu_1)^(4);

skew = mu3/sd^(3);
kurt = mu4/sd^(4);

%%%%%%%%%%%%%%%%%%
nu_all = [nu_1, nu_2, nu_3, nu_4];
disp('Non-central moments')
disp(nu_all)  % Non-central moments

mu_all = [nu_1, VAR, skew kurt];
disp('Central moments')
disp(mu_all)  % Central moments

% Graph of the density; theoretical pdf
% and compare with the graph of the theoretical solution: N(0, sigma^2/(1-rho^2))
% p = @(x) exp(-(1-rho^2)*x.^2/(2*sigma^2))/sqrt(2*pi*sigma^2/(1-rho^2));
p   = @(x) pdf('normal',x,0,sigma/sqrt(1-rho1^2));
% fplot(p,[-5,5])

x    = kmin:(kmax-kmin)/100:kmax;
ninf = norm(pk-p(x),Inf)        % norm ninf = 0.0010

%% Figure 1, 1st figure from top: Case: rho = 0.5
figure(1)
dessin1 = plot(x,pk,'r',x,nu(x),'b');
hold on 
%% legend(dessin1,'estimated invariant pdf','innovation pdf');
%% title('Graph of the pdf of the invariant probability')
hold on 
fplot(p,'g')
hold off;

Q = trapz(pk);  % Total surface below estimated pdf = 3.5572

figure(2)
dessin2 = plot(x,nu(x),'b');
hold on
fplot(p,[-5,5],'r')
hold off

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

