%%  Matlab file: pdf_SETAR211_Normal.m 
%%
%%  Coded by: JDG
%% 
%%  Computes the exact values of the mean, variance, skewness, kurtosis, 
%%  and first-lag autocorrelation function of the SETAR(2;1,1) process (31) 
%%  with N(0,1) innovations. Results are  displayed in Table 1.
%%
%%    Y_{t} =  alpha Y_{t-1}  + varepsilon_{t} if Y_{t-1} <= 0,
%%    Y_{t} = -alpha Y_{t-1}  + varepsilon_{t} if Y_{t-1} >  0.
%%
%% OUTPUT:
%% Exact pdf of SETAR(2;1,1) with N(0,1) errors and alpha_p = - alpha_m
%% alpha = 0.5
%%    nu_1,     nu_2,     nu_3,     nu_4
%%   -0.4607    1.3333   -1.6891    5.3333
%%    var,     skew,       kurt,     rho
%%    1.1211   -0.0353    3.0101    0.1729  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

alpha  = 0.5;
Qmax   = 201;
alpha2 = alpha*alpha;
alpha3 = sqrt(2*(1-alpha2)/pi);

sigma  = 1;
sigma2 = sigma*sigma;
pdf    = zeros(1,Qmax);
index  = zeros(1,Qmax);

for i=1:Qmax
  u = -10+(i-1)*0.1;
  f = sigma*alpha3*exp(-(1-alpha2)*u*u/(2*sigma2))*normcdf(-alpha*u/sigma2); % density
  pdf(i)   =  f;
  index(i) =  u;
end

figure(1)
plot(index,pdf)

mu   = -sqrt(2/pi)*alpha/sqrt(1-alpha2);  % mean, 1st central moment
mu2  = 1/(1-alpha2);                      % second central moment
var  = mu2 - mu*mu;                       % variance 
var2 = (pi-2*alpha2)/(pi*(1-alpha2));     % verification of var(u)
v2   = (1/(1-alpha2))*(1-(2*alpha2)/pi);
sd   = sqrt(v2);

ku  = (1-alpha2)/alpha2;
J_0 = sqrt(pi/(2*ku));
J_1 = -1/( ku*sqrt(1+ku) );
J_2 = J_0/ku;
J_3 = ( (2/ku)+(1/(ku+1)) )*J_1;
J_4 = (3/ku)*J_2;

nu_1 = (alpha)^(-2)*sqrt( 2* (1-alpha2)/pi )*J_1;
nu_2 = (alpha)^(-3)*sqrt( 2* (1-alpha2)/pi )*J_2;
nu_3 = (alpha)^(-4)*sqrt( 2* (1-alpha2)/pi )*J_3;
nu_4 = (alpha)^(-5)*sqrt( 2* (1-alpha2)/pi )*J_4;

%% Central moments, Andel, Netuka, Zvara (Kybernetika, 1984) 
mu3 = nu_3 - 3* nu_2 * nu_1 + 2*(nu_1)^(3);
mu4 = nu_4 - 4* nu_3 * nu_1 + 6*nu_2*(nu_1)^(2) - 3*(nu_1)^(4);

%% skewness and kurtosis
skew = mu3/sd^(3);
kurt = mu4/sd^(4);

%% ACF at lag 1
rho = ( alpha*pi + 2*alpha2*sqrt(1-alpha2) ...
       - 2*alpha2 - 2*alpha*atan(sqrt(alpha2^(-1)-1)) )/(pi-2*alpha2);

disp(' Exact pdf of SETAR(2;1,1) with N(0,1) errors and alpha_p = - alpha_m')
disp('alpha = ')
disp(alpha)
moments = [nu_1, nu_2, nu_3, nu_4];
disp('Non-central moments: nu_1, nu_2, nu_3, nu_4')
disp(moments)

var_skew_kurt = [var, skew, kurt, rho];
disp('var  skew  kurt  rho_1')
disp(var_skew_kurt)

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
