%% MATLAB File: pdf_TARMA21111.m 
%%
%% Coded by: JDG
%%
%% Computes approximate values of the  mean, variance, skewness, 
%% and kurtosis of various SETARMA(2;1,1,1,1) processes with
%% N(0,1), L(0,1), C(0,1), 
%% and Student t(3) (ST) innovations. Used for L(0,1) results in Table 4 of
%% De Gooijer (2022).
%% The exact values in Table 4 are computed with 
%% ML code: pdf_SETAR_Laplace.m. 
%%
%% Converted from R-code: TARMA-Final.R 
%% Call to file: EF.m (SEE BELOW)
%% 
%% SETARMA(2;1,1,1,1) model:
%% Y_t= [phi_1 + phi_2*Y_{t-1}+error_{t} + phi_3*error_{t-1}]I(Y_{t-1}>=r) + error_{t}
%%     +[psi_1 + psi_2*Y_{t-1}+error_{t} + psi_3*error_{t-1}]I(Y_{t-1}<r)  + error_{t}.
%% where
%% Phi = [phi_1, phi_2, phi_3];
%% Psi = [psi_1, psi_2, psi_3];
%%
%% The error distribution takes four cases:
%% error ~ N(0, sigma^2)                              <----> option==1
%% error ~ Laplace(lambda) with density 
%%         g(x) = 0.5*lambda*exp(-lambda*|x|)         <----> option==2
%% error ~ ST_3 with density 6*sqrt(3)/[pi*(3+x^2)^2] <----> option==3
%% error ~ ST_1, i.e., Cauchy distribution.           <----> option==4
%% par represents the sigma, rate and nu (degrees of freedom) 
%%
%% r     = threshold parameter
%% m     = positive integer for discarding the effect of the initial values. 
%%
%% References:
%% Li, D. and Qiu, J. (2020).
%%   The marginal density of a TMA(1) process. 
%%   Journal of Time Series Analysis, 41, 476--484.
%%
%%  De Gooijer, J.G. (2022). The marginal distribution function of
%%    threshold-type processes with central symmetric innovations,
%%    Statistics, 56.
%%
%% SOME OUTPUT of R-code:
%% Some approximate results presented in Table 4 (in parentheses)
%% of De Gooijer (2022)
%%      Phi   Psi
%% [1,] 0.00  0.00
%% [2,] 0.25 -0.25
%% [3,] 0.00  0.00
%%  nu_all:
%%       nu1_Lap  nu2_Lap  nu3_Lap  nu4_Lap   acvf_1
%% [1,] 0.2627645 2.127564 1.661292 24.95335 2.124919
%% moments:
%%       nu1_Lap      Var        skew      kurt      
%% [1,] 0.2627645   2.058519   0.006917738 5.681255 
%%
%% theoretical (exact) results:
%% case 2:
%%    0.2500
%%   -0.2500
%% case 3:
%%    0.2500
%%    g1_a,     g2_a,     g3_a,     g13_a,    a/g3_a,   h_ab,     g4_a_p2 
%%    0.9336    0.3688    2.5317    0.0156    0.0987    0.6761    0.9958
%%
%% Parameters: a (Y(t-1)>0) and b (Y(t-1)<0)
%%    0.2500   -0.2500
%% Non-central exact moments: 
%%  nu_1, nu_2, nu_3, nu_4 (Table 4)
%%  0.2633    2.1333    1.6803   25.7004
%%  Var      skewness  kurtosis  acvf_1_1  acvf_1_2 (case 1 and 2)
%%  2.0640    0.0106    5.8224    0.1383    0.1383
%%
%%    Phi Psi
%% [1,] 0.0 0.0
%% [2,] 0.5 0.1
%% [3,] 0.0 0.0
%% theoretical (exact) results:
%% case 1:
%%     0.5000
%% Parameters: a (Y(t-1)>0) and b (Y(t-1)<0)
%%    0.5000    0.1000
%% Non-central exact moments:
%%  nu_1,     nu_2,     nu_3,   nu_4
%% 0.3241    2.4072    2.5672   30.1637
%% disp(var_skew_kurt)
%%  Var     skewness  kurtosis  acvf_1_1
%%  2.3021    0.0844    5.3436    0.8792
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

Phi = [0, 0, 0.5];  %  for Y_{t-1} >= 0
Psi = [0, 0, -0.5]; %  for Y_{t-1} <  0

parameters = [Phi; Psi];

par    = 1;
option = 1; % 1 = N(0,1); 2 = L(0,1); 3 = ST(3); 4 = Cauchy = ST(1)
r      = 0;

if (Phi(2)+Phi(3)>0 & Psi(2)+Psi(3)>0)    %% case 1, 9 distinct cases in total
  p=@(x,y) EF((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),par,option) ...
     -EF(min((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
     
elseif (Phi(2)+Phi(3)>0 & Psi(2)+Psi(3)==0)          %% case 2
  p=@(x,y)  EF((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),par,option) ...
     -EF(min((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(r-y,par,option)*((x-Psi(1)-Psi(2)*y)>=0);
     
elseif (Phi(2)+Phi(3)>0 & Psi(2)+Psi(3)<0)           %% case 3
  p=@(x,y) EF((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),par,option) ...
     -EF(min((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(r-y,par,option)-EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
     
elseif (Phi(2)+Phi(3)==0 & Psi(2)+Psi(3)>0)          %% case 4
  p=@(x,y) (1-EF(r-y,par,option))*((x-Phi(1)-Phi(2)*y)>=0) ...
          + EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
          
elseif (Phi(2)+Phi(3)==0 & Psi(2)+Psi(3)==0)         %% case 5
  p=@(x,y) (1-EF(r-y,par,option))*((x-Phi(1) ...
         -Phi(2)*y)>=0)+EF(r-y,par,option)*((x-Psi(1)-Psi(2)*y)>=0);

elseif (Phi(2)+Phi(3)==0 & Psi(2)+Psi(3)<0)          %% case 6
  p=@(x,y) (1-EF(r-y,par,option))*((x-Phi(1)-Phi(2)*y)>=0) ...
     +EF(r-y,par,option)-EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
     
elseif (Phi(2)+Phi(3)<0 & Psi(2)+Psi(3)>0)           %% case 7
  p=@(x,y) 1-EF(max((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
     
elseif (Phi(2)+Phi(3)<0 & Psi(2)+Psi(3)==0)          %% case 8
  p=@(x,y) 1-EF(max((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(r-y,par,option)*((x-Psi(1)-Psi(2)*y)>=0);

else                                                 %% case 9    
  p=@(x,y) 1-EF(max((x-Phi(1)-Phi(2)*y)/(Phi(2)+Phi(3)),r-y),par,option) ...
     +EF(r-y,par,option)-EF(min((x-Psi(1)-Psi(2)*y)/(Psi(2)+Psi(3)),r-y),par,option);
end

m = 500;  
x = linspace(-10,20,m)';

xmin = min(x)-1;;
xmax = max(x)+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Construction of [m * m] matrix K
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization 
K = zeros(m,m);
for i=1:m
  K(i,1) = arrayfun(p,x(i),(x(1)+xmin)/2)   - arrayfun(p,x(i),(x(2)+x(1))/2); 
  K(i,m) = arrayfun(p,x(i),(x(m)+x(m-1))/2) - arrayfun(p,x(i),(xmax+x(m))/2);  
end

for i=1:m
  for j=2:(m-1)
    K(i,j) = arrayfun(p,x(i),(x(j)+x(j-1))/2) - arrayfun(p,x(i),(x(j+1)+x(j))/2);  
  end
end

for i=1:m                % [m * 1] vector A 
  A(i,1) = arrayfun(p,x(i),(xmax+x(m))/2);
end

H     = inv(diag(ones(m,1)) - K)*A;  % H = (I-K)^{-1} * A
tt    = linspace(-10,10,m)';        
Y_new = [tt , H];

%%%   Calculate the density ff of Y_{t}
Y1    = [0; Y_new(1:(m-1),2)];   % Y1 = x-coordinate
Y2    = [Y_new(2:m,2);1];        % Y2 = numerical CDF of delta_{t}
nr_Y  = length(Y1);
W     = Y2 - Y1;

tt2   = linspace(-10,20,m)';
tt    = linspace(-10,10,2001);

ff = zeros(m,1);
fd = zeros(2001,1);

for j=1:2001
  if (option == 1)       % N(0,1) density
    for i=1:m
      ff(i) = normpdf(tt(j)-tt2(i))*W(i);
    end
  elseif (option == 2)   % Laplace L(0,1) density
    for i=1:m
      ff(i) = 0.5*exp(-abs(tt(j)-tt2(i)))*W(i);
    end
  elseif (option == 3)   % Student t(3) density
      ff(i) = tpdf(tt(j)-tt2(i),3)*W(i);
  else                   % Cauchy = Student t(1) density 
      ff(i) = tpdf(tt(j)-tt2(i),1)*W(i);      
  end
  fd(j) = sum(ff);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot(tt,fd/2)

disp('Parameters SETARMA(2;1,1,1,1) process')
disp('Upper regime: Phi1 (constant), Phi2 (AR1), Phi3 (MA1)')
disp('Lower regime: Psi1 (constant), Psi2 (AR1), Psi3 (MA1)')
disp(parameters)

disp('Innovation density: 1 = N(0,1); 2 = L(0,1); 3 = ST_3; 4 = ST_1 (Cauchy)')
disp(option)

%% Non-central moments
nu1 = (sum(tt.*fd')/length(fd))*10;
nu2 = (sum(tt.*tt.*fd')/length(fd))*10;
nu3 = (sum(tt.*tt.*tt.*fd')/length(fd))*10;
nu4 = (sum(tt.*tt.*tt.*tt.*fd')/length(fd))*10;

nu_all = [nu1, nu2, nu3, nu4];
disp('Non-central moments: nu1, nu2, nu3, nu4')
disp(nu_all)  % Non-central moments

%% Statistics: mean, variance, skewness, kurtosis
Var  = nu2 - nu1^2;
sd   = sqrt(Var);
mu3  = nu3 - 3*nu2 * nu1 + 2*(nu1)^3;
mu4  = nu4 - 4*nu3 * nu1 + 6*nu2*(nu1)^2 - 3*(nu1)^4;
skew = mu3/sd^3;
kurt = mu4/sd^4;

mu_all = [nu1, Var, mu3, mu4, skew, kurt];
disp('Statistics: nu1=mu, Var, mu3, mu4, skewness, kurtosis')
disp(mu_all)  % Statistics based on Central moments

yt   = tt(2:length(fd));       % Y_{t-1}
yt1  = tt(1:(length(fd)-1));   % Y_{t}
fd1  = fd(2:length(fd));

%%%%%%%%%%%%%%%  END MAIN  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Cumulative distribution functions for pdf_Gauss_TARMA.m
%% file: EF.m  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [F] = EF(x,par,option)
 if (option==1)
    F = normcdf(x,0,par);                     % Normal
 elseif (option==2)
    F = 0.5+0.5*sign(x)*(1-exp(-abs(x)/par)); % Laplace
 elseif (option==3)                           % Student t3
    F = tcdf(x,par); 
 else F = tcdf(x,par);                        % Student t1, par=1
 end


